/*************************************************************************
  (c) Copyright.  Digital Equipment Corporation, 1995.  All Rights
  Reserved.

  Permission is hereby granted to use, copy, modify, or enhance this 
  software freely, as long as the foregoing copyright of Digital Equipment
  Corporation and this notice are retained on the software.  This 
  software may not be distributed or sublicensed for a fee.  Digital      
  makes this software available "AS IS" and without warranties of any
  kind.  
 *************************************************************************/

/*
* Module Name:
*   utl_os.h
*   
* Abstract:
*   This is the interface for the basic functions that are provided by the OS.
*
* Authors:
*   DMW - Douglas M. Washabaugh
*
* Modification History:
*   Date       Name  Description 
*   08-Aug-94  DMW   Created.
*   02-Nov-94  DMW   Updated to use handles, added timer routines.
*   10-Nov-94  DMW   Updated with error log and print routines.
*   01-Jan-95  DMW   Allocated memory is now zeroed.
*   10-Jan-95  DMW   Remove obsolete phys mem routines, added mem statistics.
*   03-Feb-95  PJR   Added test string input to os_create.
*   16-Mar-95  DMW   Added two new routines os_buff_hdr_copy, os_buff_hdr_get.
*
* Description:
*   There will be one instantiation of this module for each adapter in the 
*   system.
*
* Call Sequence:
*   The following shows a rough sequence of how calls can be made to this
*   module.  Because this module is a collection of miscellaneous routines,
*   a different method is used to represent the call sequence.  The method
*   says that a given routine cannot be called unless all routines with 
*   a smaller indention are called first.  For example, the following shows
*   that os_timer_cancel cannot be called until os_timer_set is called, which
*   cannot be called before os_timer_allocate, which cannot be called before
*   os_create.
*
*     - os_create
*        - os_timer_allocate
*           - os_timer_set
*              - os_timer_cancel
*           - os_timer_deallocate
*        - os_reg_get
*        - os_reg_set
*        - os_mem_alloc
*           - os_mem_dealloc
*        - os_phys_mem_alloc
*           - os_phys_mem_dealloc
*        - os_print
*        - os_err_log
*        - os_destroy   
*/

/*++
* ---------------------
* - OS_TIMER_CALLBACK -
* ---------------------
*
* Overview:
*   Routine that is called when a timer expires.
*
* Arguments:
*   context - (IN) The value that was specified when the timer was allocated.
*   
* Returns:
*   none
*
* Preconditions:
*   None
*
* Postconditions:
*   none
--*/

typedef void (*OS_TIMER_CALLBACK) (HANDLE context);

/*++
* =============
* = os_create =
* =============
*
* Overview:
*   Creates an instance of this module.
*
* Arguments:
*   adap_handle        - (IN)  The handle that is passed to the operating system
*                              when calls are made.  For example, when memory is
*                              allocated, this handle is passed.  Typically this
*                              is an adapter specific handle, but it could also 
*                              be a per driver handle.
*
*   p_text             - (IN)  Pointer to a text string to identify this 
*                              instance.
*
*   p_os_handle        - (OUT) This handle must be passed to subsequent calls to
*                              this module.  The information contained in this
*                              handle is implementation dependent, and may not 
*                              be used outside of this module.
*  
*   
* Returns:
*   STATUS_K_SUCCESS   - Success
*   STATUS_K_RESOURCES - Memory could not be allocated, instance cannot be used.
*
* Preconditions:
*   None
*
* Postconditions:
*   - An instantiation was created and can now be used.  This instantiation must
*     later be "destroyed".
*
* Issues:
*   None
--*/

STATUS os_create (HANDLE  adap_handle,
                  char   *p_text,
                  HANDLE *p_os_handle);

/*++
* ==============
* = os_destroy =
* ==============
*
* Overview:
*   Destroys an instance of this module.  All allocated memory must be
*   deallocated before this routine is called.  All timers are canceled and 
*   deallocated.
*
*   Note that it is possible that some timers can not be canceled, and
*   that the actual destruction of this instantiation may occur when the final
*   timer has expired.  Regardless, this routine returns immediately and the
*   caller can consider the instantiation destroyed.
*
* Arguments:
*   os_handle - (IN) The handle that was returned from os_create. 
*   
* Returns:
*   none
*
* Preconditions:
*   none
*
* Postconditions:
*   No more calls can be made to this instantiation.
--*/

void os_destroy (HANDLE os_handle);

/*++
* ============
* = os_delay =
* ============
*
* Overview:
*   Delays the specified number of microseconds.  This should not be called
*   often or with large values.  In most implementations, the processor
*   enters a tight software loop.
*
* Arguments:
*  os_handle - (IN)  The handle returned from os_create.
*
*  delay     - (IN)  The number of microseconds to delay.
*
* Returns:
*   none
*
* Preconditions:
*   none
*
* Postconditions:
*   A small amount of time may have passed.
--*/

void os_delay (HANDLE os_handle,
               UINT32 delay);

/*++
* ==================
* = os_timer_alloc =
* ==================
*
* Overview:
*   Allocates and initializes a timer.  Some implementations of this
*   routine may allocate memory or other resources.  This routine can be
*   called from the coded path that processes receives or "transmit dones"
*   (which may include interrupt level for some operating systems).
*
* Arguments:
*  os_handle      - (IN)  The handle returned from os_create.
*
*  callback       - (IN)  Pointer to the routine to callback when the timer
*                         expires.
*
*  context        - (IN)  Value to pass to the callback routine.
*
*  p_timer_handle - (OUT) Pointer to a timer handle.  This value must be used
*                         when setting, canceling, or deallocating a timer.
*                         It will have the os_handle associated with it so that
*                         if the timer is specified in a call, the os_handle
*                         does not also need to be specified.
* Returns:
*   STATUS_K_SUCCESS   - Timer was successfully allocated
*   STATUS_K_RESOURCES - Timer was NOT allocated.
*
* Preconditions:
*   none
*
* Postconditions:
*   A timer may have been allocated.
--*/
STATUS os_timer_alloc (HANDLE             os_handle,
                       OS_TIMER_CALLBACK  callback,
                       HANDLE             context,
                       HANDLE            *p_timer_handle);
/*++
* ================
* = os_timer_set =
* ================
*
* Overview:
*   Sets a timer to expire after a specified time.  When the timer expires,
*   its associated callback routine is called.
*
* Arguments:
*  timer_handle - (IN)  The value returned from os_timer_alloc.
*
*  time         - (IN)  Number of milliseconds until the callback is called.
*
* Returns:
*   STATUS_K_SUCCESS
*   STATUS_K_FAILURE
*
* Preconditions:
*   - The timer was allocated.
*
* Postconditions:
*   A timer is set.                    
--*/
STATUS os_timer_set (HANDLE timer_handle,
                     UINT32  time);

/*++
* ===================
* = os_timer_cancel =
* ===================
*
* Overview:
*   Cancels the specified timer.   The timer callback routine is guaranteed
*   not be called after this routine completes.  Also, it is permissible to
*   cancel a timer that has not been set.
*
* Arguments:
*  timer_handle - (IN)  The value returned from os_timer_alloc.
*
* Returns:
*   STATUS_K_SUCCESS       - Timer was successfully canceled.
*
*   STATUS_K_NO_WORK_TO_DO - Timer was not set.
*
* Preconditions:
*   - The timer was allocated.
*
* Postconditions:
*   - The timer is canceled.
*   - The timer callback will not be called.                    
--*/
STATUS os_timer_cancel (HANDLE timer_handle);


/*++
* ====================
* = os_timer_dealloc =
* ====================
*
* Overview:
*   Deallocates the specified timer.  Even though some implementations may not 
*   support this feature, it must be called.  Note that this routine can be
*   called even if the timer has been started, but not yet expired.
*
*   After this routine completes, it is guaranteed that the timer callback will
*   not be called.
*
* Arguments:
*  timer_handle - (IN)  The value returned from os_timer_alloc.
*
* Returns:
*   none
*
* Preconditions:
*   - The timer was allocated.
*
* Postconditions:
*   - A timer was deallocated
*   - The timer callback will not be called.
--*/
void os_timer_dealloc (HANDLE timer_handle);


/*++
* ==============
* = os_reg_set =
* ==============
*
* Overview:
*   Writes the specified longword value to the specified adapter register.
*
* Arguments:
*  os_handle  - (IN) The handle returned from os_create.
*
*  p_reg      - (IN) Pointer to register to write.  Note that this pointer
*                    can be relative to the base of the registers, depending
*                    upon the implementation.
*
*  value      - (IN) Longword value to write to the register.
*
* Returns:
*   None
*
* Preconditions:
*   - The driver has been sufficiently initialized so that it is ready to
*     handle reads from adapter registers.
*
* Postconditions:
*   The value of the specified register is set.                    
--*/
void os_reg_set (HANDLE   os_handle,
                 UINT32  *p_reg,
                 UINT32   value);



/*++
* ==============
* = os_reg_get =
* ==============
*
* Overview:
*   Reads the contents of the specified longword adapter register.
*
* Arguments:
*  os_handle  - (IN)  The value that was returned by os_create.
*
*  p_reg      - (IN)  Relative pointer to the register to read.
*
* Returns:
*   The value of the register.
*
* Preconditions:
*   - The driver has been sufficiently initialized so that it is ready to
*     handle reads from adapter registers.
*
* Postconditions:
*   None
--*/

UINT32 os_reg_get (HANDLE   os_handle, 
                   UINT32  *p_reg);



/*++
* ================
* = os_mem_alloc =
* ================
*
* Overview:
*   Allocates the specified number of bytes from virtual memory, and returns
*   the virtual address of the memory.  The allocated memory is zeroed.
*
*   This routine can be called from the coded path that processes receives or
*   "transmit dones" (which may include interrupt level for some operating
*   systems).
*
* Arguments:
*   os_handle - (IN) The handle that was returned by os_create.
*
*   size      - (IN) The number of bytes to allocate.
*  
* Returns:
*   - Null, if call failed, else pointer to allocated memory.
*
* Preconditions:
*   None
*
* Postconditions:
*   None
--*/

void *os_mem_alloc (HANDLE os_handle, 
                    UINT32 size);

/*++
* ==================
* = os_mem_dealloc =
* ==================
*
* Overview:
*   Deallocates the specified virtual memory.  This routine must not be called
*   from interrupt level.
*
* Arguments:
*   os_handle - (IN) The handle that was returned by os_create.
* 
*   p_mem     - (IN) Pointer to the memory to return.
*
* Returns:
*   None
*
* Preconditions:
*   None
*
* Postconditions:
*   None
--*/

void os_mem_dealloc (HANDLE  os_handle,
                     void   *p_mem);

/*++
* =================
* = os_mem_config =
* =================
*
* Overview:
*   Specifies the number of bytes that are availble in the "memory pool".  Any 
*   attempts to allocate more than this memory result in an allocation failure.
*   This number is the sum for all allocations, not just the maximum for any
*   given allocation.
*
*   Before setting this value, the caller should call os_mem_stats to determine
*   how much memory is already allocated.  For example, if a user wants to 
*   limit the amount there code allocates, it should add that number to how
*   much is already allocated and add that to the pool_size.
*
*   This routine is for testing only, and should not be implemented by driver
*   implementations.  This routine may only be called by test code.
*
* Arguments:
*   os_handle - (IN) The handle that was returned by os_create.
* 
*   pool_size - (IN) The number of bytes available for allocation. 
*
* Returns:
*   None
*
* Preconditions:
*   - Previously allocated memory is not included in the calculations.
*
* Postconditions:
*   - A limit is placed on memory allocations.
--*/

void os_mem_config (HANDLE  os_handle,
                    UINT32  pool_size);

/*++
* ================
* = os_mem_stats =
* ================
*
* Overview:
*   Returns statistics relating to memory usage.  When this routine is first
*   called it will return a non-zero number because this module itself
*   allocates some memory.  Therefore, to determine how much memory the 
*   caller's code allocates, the caller must call this routine before and
*   after allocating memory, and subtracting the results.
*
*   This routine is for testing only, and should not be implemented by driver
*   implementations.  This routine may only be called by test code.
*
* Arguments:
*   os_handle         - (IN)  The handle that was returned by os_create.
* 
*   p_peak_bytes      - (OUT) Pointer to where the peak number of allocated bytes
*                             is written.  
*
*   p_allocated_bytes - (OUT) Pointer to where the number of currently allocated
*                             bytes is written.
* Returns:
*   None
*
* Preconditions:
*   none
*
* Postconditions:
*   none
--*/

void os_mem_stats (HANDLE  os_handle,
                   UINT32  *p_peak_bytes,
                   UINT32  *p_allocated_bytes);

/*++
* ============
* = os_print =
* ============
*
* Overview:
*   Prints the specified string to a display visible to users.  This routine 
*   may be a "null" implementation for some operating systems.
*
* Arguments:
*   p_string  - (IN) Pointer to null-terminated string to print.
*
* Returns:
*   None
*
* Preconditions:
*   None
*
* Postconditions:
*   None
--*/

void os_print (void   *p_string);

/*++
* ===================
* = os_buff_hdr_get =
* ===================
*
* Overview:
*   Copies the first two bytes from a packet (whose internal format is not 
*   known to the common code) to the LEC header.
*
*   The caller can determine that the entire LEC header was copied by ensuring
*   that the value returned for bytes copied matches the size of the LEC 
*   header.
*
* Arguments:
*   p_pkt          - (IN) A pointer to the packet from which to retrieve the
*                         LEC header.
* 
*   p_lec_hdr      - (OUT) The value copied from the first two bytes of the
*                          packet.
*
*   p_bytes_copied - (OUT) The number of bytes copied.
*
* Returns:
*   None
*
* Preconditions:
*   - p_lec_hdr must point to a buffer large enough to hold the LEC header.
*
* Postconditions:
*   - p_bytes_copied has the number of bytes copied
*   - p_bytes_copied == 0 if the pkt length is '0' or the packet is corrupt.
*   - p_lec_hdr contains the bytes copied from the packet.
--*/

void os_buff_hdr_get (void   *p_pkt,
                      UINT16 *p_lec_hdr,
                      UINT32 *p_bytes_copied);

/*++
* ====================
* = os_buff_hdr_copy =
* ====================
*
* Overview:
*   Copies the specified packet (whose internal format is not known to the
*   common code) to the specified buffer.  If the packet is larger than the 
*   destination buffer, then only the part of the packet which fits is 
*   copied.  The actual number of bytes copied is returned.  If the packet is
*   corrupt, than the value returned for the number of bytes copied is 0.
*
*   The caller can determine that the entire 
*
* Arguments:
*   p_pkt          - (IN) A pointer to the packet to copy.
* 
*   pkt_len        - (IN)  Number of bytes in the buffer pointed to by p_pkt.
*                          This argument may be ignored for implementations
*                          whose packet structure contains the packet length.
*                          Note that callers must always provide this value.
*
*   p_dst          - (OUT) Pointer to the destination buffer.
*
*   dst_len        - (IN)  Number of bytes in the buffer pointed to by p_dst.
*
*   p_bytes_copied - (OUT) Number of bytes copied to p_dst.
*
* Returns:
*   None
*
* Preconditions:
*   None
*
* Postconditions:
*   - p_bytes_copied has the number of bytes copied
*   - p_bytes_copied is <= dst_len
*   - p_bytes_copied == 0 if the pkt length is '0' or the packet is corrupt.
*   - p_dst contains the bytes copied from the packet.
--*/

void os_buff_hdr_copy (void   *p_pkt,
                       UINT32  pkt_len,
                       void   *p_dst,
                       UINT32  dst_len,
                       UINT32 *p_bytes_copied);

